@@ -351,7 +351,6 @@ webkit_pref_callback_gnome_fonts (GSettings  *ephy_settings,
   if (g_settings_get_boolean (ephy_settings, key)) {
     g_object_set (webkit_settings,
                   "default-font-family", "serif",
-                  "serif-font-family", "serif",
                   "sans-serif-font-family", "sans-serif",
                   "monospace-font-family", "monospace",
                   "default-font-size", webkit_settings_font_size_to_pixels (12),
@@ -475,11 +474,6 @@ static const PrefData webkit_pref_entries[] = {
     EPHY_PREFS_WEB_USER_AGENT,
     "user-agent",
     webkit_pref_callback_user_agent },
-  { EPHY_PREFS_WEB_SCHEMA,
-    EPHY_PREFS_WEB_MOBILE_USER_AGENT,
-    "mobile-user-agent",
-    /* No need for a specific callback, we just want to update the user agent. */
-    webkit_pref_callback_user_agent },
   { EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_COOKIES_POLICY,
     "accept-policy",
