@@ -85,7 +85,7 @@ void                       ephy_web_view_load_request             (EphyWebView
 void                       ephy_web_view_load_url                 (EphyWebView               *view,
                                                                    const char                *url);
 gboolean                   ephy_web_view_is_loading               (EphyWebView               *view);
-gboolean                   ephy_web_view_get_load_failed          (EphyWebView               *view);
+gboolean                   ephy_web_view_load_failed              (EphyWebView               *view);
 GdkPixbuf *                ephy_web_view_get_icon                 (EphyWebView               *view);
 EphyWebViewDocumentType    ephy_web_view_get_document_type        (EphyWebView               *view);
 EphyWebViewNavigationFlags ephy_web_view_get_navigation_flags     (EphyWebView               *view);
@@ -143,6 +143,13 @@ void                       ephy_web_view_get_web_app_title        (EphyWebView
 char                      *ephy_web_view_get_web_app_title_finish (EphyWebView               *view,
                                                                    GAsyncResult              *result,
                                                                    GError                   **error);
+void                       ephy_web_view_get_web_app_mobile_capable        (EphyWebView         *view,
+                                                                            GCancellable        *cancellable,
+                                                                            GAsyncReadyCallback  callback,
+                                                                            gpointer             user_data);
+gboolean                   ephy_web_view_get_web_app_mobile_capable_finish (EphyWebView   *view,
+                                                                            GAsyncResult  *result,
+                                                                            GError       **error);
 
 void                       ephy_web_view_set_visit_type           (EphyWebView *view, 
                                                                    EphyHistoryPageVisitType visit_type);
