@@ -105,16 +105,11 @@ static const char * const ephy_prefs_state_schema[] = {
 #define EPHY_PREFS_WEB_MONOSPACE_FONT               "monospace-font"
 #define EPHY_PREFS_WEB_ENABLE_USER_CSS              "enable-user-css"
 #define EPHY_PREFS_WEB_ENABLE_POPUPS                "enable-popups"
-#define EPHY_PREFS_WEB_ENABLE_PLUGINS               "enable-plugins"
 #define EPHY_PREFS_WEB_ENABLE_SPELL_CHECKING        "enable-spell-checking"
-#define EPHY_PREFS_WEB_ENABLE_WEBGL                 "enable-webgl"
-#define EPHY_PREFS_WEB_ENABLE_WEBAUDIO              "enable-webaudio"
 #define EPHY_PREFS_WEB_ENABLE_SMOOTH_SCROLLING      "enable-smooth-scrolling"
 #define EPHY_PREFS_WEB_USER_AGENT                   "user-agent"
-#define EPHY_PREFS_WEB_MOBILE_USER_AGENT            "mobile-user-agent"
 #define EPHY_PREFS_WEB_COOKIES_POLICY               "cookies-policy"
 #define EPHY_PREFS_WEB_DEFAULT_ENCODING             "default-encoding"
-#define EPHY_PREFS_WEB_DO_NOT_TRACK                 "do-not-track"
 #define EPHY_PREFS_WEB_ENABLE_ADBLOCK               "enable-adblock"
 #define EPHY_PREFS_WEB_REMEMBER_PASSWORDS           "remember-passwords"
 #define EPHY_PREFS_WEB_ENABLE_SITE_SPECIFIC_QUIRKS  "enable-site-specific-quirks"
@@ -137,16 +132,11 @@ static const char * const ephy_prefs_web_schema[] = {
   EPHY_PREFS_WEB_MONOSPACE_FONT,
   EPHY_PREFS_WEB_ENABLE_USER_CSS,
   EPHY_PREFS_WEB_ENABLE_POPUPS,
-  EPHY_PREFS_WEB_ENABLE_PLUGINS,
   EPHY_PREFS_WEB_ENABLE_SPELL_CHECKING,
-  EPHY_PREFS_WEB_ENABLE_WEBGL,
-  EPHY_PREFS_WEB_ENABLE_WEBAUDIO,
   EPHY_PREFS_WEB_ENABLE_SMOOTH_SCROLLING,
   EPHY_PREFS_WEB_USER_AGENT,
-  EPHY_PREFS_WEB_MOBILE_USER_AGENT,
   EPHY_PREFS_WEB_COOKIES_POLICY,
   EPHY_PREFS_WEB_DEFAULT_ENCODING,
-  EPHY_PREFS_WEB_DO_NOT_TRACK,
   EPHY_PREFS_WEB_ENABLE_ADBLOCK,
   EPHY_PREFS_WEB_REMEMBER_PASSWORDS,
   EPHY_PREFS_WEB_ENABLE_SITE_SPECIFIC_QUIRKS,
@@ -166,7 +156,6 @@ static const char * const ephy_prefs_web_schema[] = {
 #define EPHY_PREFS_NEW_WINDOWS_IN_TABS                "new-windows-in-tabs"
 #define EPHY_PREFS_WARN_ON_CLOSE_UNSUBMITTED_DATA     "warn-on-close-unsubmitted-data"
 #define EPHY_PREFS_ENABLE_CARET_BROWSING              "enable-caret-browsing"
-#define EPHY_PREFS_INTERNAL_VIEW_SOURCE               "internal-view-source"
 #define EPHY_PREFS_RESTORE_SESSION_POLICY             "restore-session-policy"
 #define EPHY_PREFS_RESTORE_SESSION_DELAYING_LOADS     "restore-session-delaying-loads"
 #define EPHY_PREFS_CONTENT_FILTERS                    "content-filters"
@@ -183,6 +172,7 @@ static const char * const ephy_prefs_web_schema[] = {
 #define EPHY_PREFS_LOCKDOWN_SAVE_TO_DISK      "disable-save-to-disk"
 #define EPHY_PREFS_LOCKDOWN_PRINTING          "disable-printing"
 #define EPHY_PREFS_LOCKDOWN_QUIT              "disable-quit"
+#define EPHY_PREFS_LOCKDOWN_CONTEXT_MENU      "disable-context-menu"
 
 #define EPHY_PREFS_SYNC_SCHEMA            "org.gnome.Epiphany.sync"
 #define EPHY_PREFS_SYNC_USER              "sync-user"
@@ -205,6 +195,7 @@ static const char * const ephy_prefs_web_schema[] = {
 
 #define EPHY_PREFS_WEB_APP_SCHEMA          "org.gnome.Epiphany.webapp"
 #define EPHY_PREFS_WEB_APP_ADDITIONAL_URLS "additional-urls"
+#define EPHY_PREFS_WEB_APP_MOBILE_CAPABLE  "mobile-capable"
 
 static struct {
   const char *schema;
