@@ -363,16 +363,18 @@ create_desktop_file (const char *id,
  * @address: the address of the new web application
  * @name: the name for the new web application
  * @icon: the icon for the new web application
+ * @options: the options for the new web application
  *
  * Creates a new Web Application for @address.
  *
  * Returns: (transfer-full): the path to the desktop file representing the new application
  **/
 char *
-ephy_web_application_create (const char *id,
-                             const char *address,
-                             const char *name,
-                             GdkPixbuf  *icon)
+ephy_web_application_create (const char                *id,
+                             const char                *address,
+                             const char                *name,
+                             GdkPixbuf                 *icon,
+                             EphyWebApplicationOptions  options)
 {
   g_autofree char *app_file = NULL;
   g_autofree char *profile_dir = NULL;
@@ -408,7 +410,7 @@ ephy_web_application_create (const char *id,
   /* Create the deskop file. */
   desktop_file_path = create_desktop_file (id, name, address, profile_dir, icon);
   if (desktop_file_path)
-    ephy_web_application_initialize_settings (profile_dir);
+    ephy_web_application_initialize_settings (profile_dir, options);
 
   return g_steal_pointer (&desktop_file_path);
 }
@@ -508,7 +510,6 @@ ephy_web_application_setup_from_desktop_file (GDesktopAppInfo *desktop_info)
     if (names)
       gtk_window_set_default_icon_name (names[0]);
   }
-  g_clear_object (&icon);
 
   /* We need to re-set this because we have already parsed the
    * options, which inits GTK+ and sets this as a side effect.
@@ -547,7 +548,7 @@ ephy_web_application_for_profile_directory (const char *profile_dir)
 
   id = get_app_id_from_profile_directory (profile_dir);
   if (!id)
-    return NULL;
+    g_error ("Cannot create EphyWebApplication: failed to get app ID from profile directory %s", profile_dir);
 
   app = g_new0 (EphyWebApplication, 1);
   app->id = g_strdup (id);
@@ -555,11 +556,8 @@ ephy_web_application_for_profile_directory (const char *profile_dir)
   app->desktop_file = get_app_desktop_filename (id);
   desktop_file_path = g_build_filename (profile_dir, app->desktop_file, NULL);
   desktop_info = g_desktop_app_info_new_from_filename (desktop_file_path);
-  if (!desktop_info) {
-    ephy_web_application_free (app);
-    g_free (desktop_file_path);
-    return NULL;
-  }
+  if (!desktop_info)
+    g_error ("Cannot create EphyWebApplication: failed to create GDesktopAppInfo from desktop file %s", desktop_file_path);
 
   app->name = g_strdup (g_app_info_get_name (G_APP_INFO (desktop_info)));
   app->icon_url = g_desktop_app_info_get_string (desktop_info, "Icon");
@@ -712,7 +710,8 @@ ephy_web_application_exists (const char *id)
 }
 
 void
-ephy_web_application_initialize_settings (const char *profile_directory)
+ephy_web_application_initialize_settings (const char                *profile_directory,
+                                          EphyWebApplicationOptions  options)
 {
   GSettings *settings;
   GSettings *web_app_settings;
@@ -753,6 +752,16 @@ ephy_web_application_initialize_settings (const char *profile_directory)
 
   g_object_unref (settings);
   g_object_unref (web_app_settings);
+
+  if (options & EPHY_WEB_APPLICATION_MOBILE_CAPABLE) {
+    path = g_build_path ("/", "/org/gnome/epiphany/web-apps/", name, "webapp/", NULL);
+    web_app_settings = g_settings_new_with_path (EPHY_PREFS_WEB_APP_SCHEMA, path);
+    g_free (path);
+
+    g_settings_set_boolean (web_app_settings, EPHY_PREFS_WEB_APP_MOBILE_CAPABLE, TRUE);
+    g_object_unref (web_app_settings);
+  }
+
   g_free (name);
 }
 
