@@ -27,7 +27,6 @@
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-utils.h"
 #include "ephy-favicon-helpers.h"
-#include "ephy-gui.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-window.h"
@@ -308,13 +307,13 @@ navigation_button_press_event_cb (GtkButton *button,
 {
   EphyActionBarStart *action_bar_start = EPHY_ACTION_BAR_START (user_data);
   EphyNavigationHistoryDirection direction;
+  const gchar *action_name;
   PopupData *data;
-  gboolean is_back = FALSE;
 
-  is_back = (GTK_WIDGET (button) == action_bar_start->navigation_back);
+  action_name = gtk_actionable_get_action_name (GTK_ACTIONABLE (button));
 
-  direction = is_back ? EPHY_NAVIGATION_HISTORY_DIRECTION_BACK
-                      : EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD;
+  direction = strstr (action_name, "back") ? EPHY_NAVIGATION_HISTORY_DIRECTION_BACK
+                                           : EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD;
 
   switch (((GdkEventButton *)event)->button) {
     case GDK_BUTTON_SECONDARY:
@@ -355,49 +354,30 @@ navigation_button_release_event_cb (GtkButton *button,
   GActionGroup *action_group;
   GAction *action;
   EphyNavigationHistoryDirection direction;
-  gboolean is_back = FALSE;
-  gboolean open_in_new_tab = FALSE;
-  gboolean open_in_current_tab = FALSE;
-  GdkEventType type = GDK_NOTHING;
-  guint state = 0, button_val = (guint) - 1, keyval = (guint) - 1;
-
-  ephy_gui_get_current_event (&type, &state, &button_val, &keyval);
-  is_back = (GTK_WIDGET (button) == action_bar_start->navigation_back);
+  const gchar *action_name;
 
   g_clear_handle_id (&action_bar_start->navigation_buttons_menu_timeout, g_source_remove);
 
+  action_name = gtk_actionable_get_action_name (GTK_ACTIONABLE (button));
   action_group = gtk_widget_get_action_group (gtk_widget_get_ancestor (GTK_WIDGET (action_bar_start), EPHY_TYPE_WINDOW), "toolbar");
 
-  direction = is_back ? EPHY_NAVIGATION_HISTORY_DIRECTION_BACK
-                      : EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD;
+  direction = strcmp (action_name, "toolbar.navigation-back") == 0 ? EPHY_NAVIGATION_HISTORY_DIRECTION_BACK
+                                                                   : EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD;
 
-  open_in_new_tab = (((GdkEventButton *)event)->button == GDK_BUTTON_MIDDLE) || (state == GDK_CONTROL_MASK);
-  open_in_current_tab = ((GdkEventButton *)event)->button == GDK_BUTTON_PRIMARY;
-
-  if (open_in_new_tab) {
-    if (direction == EPHY_NAVIGATION_HISTORY_DIRECTION_BACK) {
-      action = g_action_map_lookup_action (G_ACTION_MAP (action_group),
-                                           "navigation-back-new-tab");
-      g_action_activate (action, NULL);
-    } else if (direction == EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD) {
-      action = g_action_map_lookup_action (G_ACTION_MAP (action_group),
-                                           "navigation-forward-new-tab");
-      g_action_activate (action, NULL);
-    }
-    return GDK_EVENT_STOP;
-  }
-
-  if (open_in_current_tab) {
-    if (direction == EPHY_NAVIGATION_HISTORY_DIRECTION_BACK) {
-      action = g_action_map_lookup_action (G_ACTION_MAP (action_group),
-                                           "navigation-back");
-      g_action_activate (action, NULL);
-    } else if (direction == EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD) {
-      action = g_action_map_lookup_action (G_ACTION_MAP (action_group),
-                                           "navigation-forward");
-      g_action_activate (action, NULL);
-    }
-    return GDK_EVENT_STOP;
+  switch (((GdkEventButton *)event)->button) {
+    case GDK_BUTTON_MIDDLE:
+      if (direction == EPHY_NAVIGATION_HISTORY_DIRECTION_BACK) {
+        action = g_action_map_lookup_action (G_ACTION_MAP (action_group),
+                                             "navigation-back-new-tab");
+        g_action_activate (action, NULL);
+      } else if (direction == EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD) {
+        action = g_action_map_lookup_action (G_ACTION_MAP (action_group),
+                                             "navigation-forward-new-tab");
+        g_action_activate (action, NULL);
+      }
+      return GDK_EVENT_STOP;
+    default:
+      break;
   }
 
   return GDK_EVENT_PROPAGATE;
@@ -486,15 +466,6 @@ ephy_action_bar_start_dispose (GObject *object)
   G_OBJECT_CLASS (ephy_action_bar_start_parent_class)->dispose (object);
 }
 
-static GtkIconSize
-get_icon_size (void)
-{
-  if (is_desktop_pantheon ())
-    return GTK_ICON_SIZE_LARGE_TOOLBAR;
-
-  return GTK_ICON_SIZE_BUTTON;
-}
-
 static void
 update_new_tab_button_visibility (EphyActionBarStart *action_bar_start)
 {
@@ -663,3 +634,25 @@ ephy_action_bar_start_get_placeholder (EphyActionBarStart *action_bar_start)
 {
   return action_bar_start->placeholder;
 }
+
+void
+ephy_action_bar_start_set_adaptive_mode (EphyActionBarStart *action_bar,
+                                         EphyAdaptiveMode    adaptive_mode)
+{
+  GValue val = G_VALUE_INIT;
+
+  g_value_init (&val, G_TYPE_INT);
+
+  gtk_widget_set_visible (action_bar->new_tab_button, adaptive_mode == EPHY_ADAPTIVE_MODE_NORMAL);
+  gtk_widget_set_visible (action_bar->combined_stop_reload_button, adaptive_mode == EPHY_ADAPTIVE_MODE_NORMAL);
+
+  if (adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW)
+    g_value_set_int (&val, 42);
+  else
+    g_value_set_int (&val, -1);
+
+  g_object_set_property (G_OBJECT (action_bar->navigation_back), "width-request", &val);
+  g_object_set_property (G_OBJECT (action_bar->navigation_forward), "width-request", &val);
+
+  g_value_unset (&val);
+}
