@@ -26,9 +26,11 @@
 #include "ephy-add-bookmark-popover.h"
 #include "ephy-desktop-utils.h"
 #include "ephy-embed-utils.h"
+#include "ephy-file-helpers.h"
 #include "ephy-flatpak-utils.h"
 #include "ephy-location-entry.h"
 #include "ephy-notebook.h"
+#include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-title-box.h"
 #include "ephy-title-widget.h"
@@ -46,6 +48,9 @@ enum {
 
 static GParamSpec *object_properties[N_PROPERTIES] = { NULL, };
 
+/* Translators: tooltip for the refresh button */
+static const char *REFRESH_BUTTON_TOOLTIP = N_("Reload the current page");
+
 struct _EphyHeaderBar {
   GtkHeaderBar parent_instance;
 
@@ -58,6 +63,8 @@ struct _EphyHeaderBar {
   GtkWidget *page_menu_button;
   GtkWidget *zoom_level_button;
   GtkWidget *restore_button;
+  GtkWidget *combined_stop_reload_button;
+  GtkWidget *combined_stop_reload_image;
 };
 
 G_DEFINE_TYPE (EphyHeaderBar, ephy_header_bar, GTK_TYPE_HEADER_BAR)
@@ -158,36 +165,6 @@ update_revealer_visibility (GtkRevealer *revealer)
                           gtk_revealer_get_child_revealed (revealer));
 }
 
-static void
-handle_primary_tab_key (GtkWidget *widget,
-                        gpointer   user_data)
-{
-  EphyHeaderBar *header_bar = EPHY_HEADER_BAR (user_data);
-  GtkWidget *entry = ephy_location_entry_get_entry (EPHY_LOCATION_ENTRY (header_bar->title_widget));
-  GtkWidget *nb;
-
-  nb = ephy_window_get_notebook (header_bar->window);
-  g_assert (nb != NULL);
-
-  ephy_notebook_next_page (EPHY_NOTEBOOK (nb));
-  gtk_widget_grab_focus (entry);
-}
-
-static void
-handle_primary_iso_tab_key (GtkWidget *widget,
-                            gpointer   user_data)
-{
-  EphyHeaderBar *header_bar = EPHY_HEADER_BAR (user_data);
-  GtkWidget *entry = ephy_location_entry_get_entry (EPHY_LOCATION_ENTRY (header_bar->title_widget));
-  GtkWidget *nb;
-
-  nb = ephy_window_get_notebook (header_bar->window);
-  g_assert (nb != NULL);
-
-  ephy_notebook_prev_page (EPHY_NOTEBOOK (nb));
-  gtk_widget_grab_focus (entry);
-}
-
 static void
 ephy_header_bar_constructed (GObject *object)
 {
@@ -227,28 +204,7 @@ ephy_header_bar_constructed (GObject *object)
   if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_APPLICATION)
     header_bar->title_widget = EPHY_TITLE_WIDGET (ephy_title_box_new ());
   else {
-    DzlShortcutController *controller;
-    GtkWidget *entry;
-
     header_bar->title_widget = EPHY_TITLE_WIDGET (ephy_location_entry_new ());
-    entry = ephy_location_entry_get_entry (EPHY_LOCATION_ENTRY (header_bar->title_widget));
-    controller = dzl_shortcut_controller_find (entry);
-
-    dzl_shortcut_controller_add_command_callback (controller,
-                                                  "org.gnome.Epiphany.next-tab-pages",
-                                                  "<Primary>Tab",
-                                                  DZL_SHORTCUT_PHASE_DISPATCH,
-                                                  handle_primary_tab_key,
-                                                  header_bar,
-                                                  NULL);
-
-    dzl_shortcut_controller_add_command_callback (controller,
-                                                  "org.gnome.Epiphany.prev-tab-pages",
-                                                  "<Primary>ISO_Left_Tab",
-                                                  DZL_SHORTCUT_PHASE_DISPATCH,
-                                                  handle_primary_iso_tab_key,
-                                                  header_bar,
-                                                  NULL);
   }
 
   if (is_desktop_pantheon ()) {
@@ -328,6 +284,10 @@ ephy_header_bar_constructed (GObject *object)
       gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "help-button")));
   }
 
+  header_bar->combined_stop_reload_button = GTK_WIDGET (gtk_builder_get_object (builder, "combined_stop_reload_button"));
+  header_bar->combined_stop_reload_image = GTK_WIDGET (gtk_builder_get_object (builder, "combined_stop_reload_image"));
+  gtk_widget_set_tooltip_text (header_bar->combined_stop_reload_button, _(REFRESH_BUTTON_TOOLTIP));
+
   if (is_desktop_pantheon ()) {
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "about-button")));
 
@@ -362,6 +322,12 @@ ephy_header_bar_constructed (GObject *object)
   gtk_size_group_add_widget (downloads_size_group,
                              ephy_action_bar_end_get_downloads_revealer (header_bar->action_bar_end));
   g_object_unref (downloads_size_group);
+
+  if (ephy_profile_dir_is_web_application ()) {
+    GtkWidget *navigation_box = ephy_action_bar_start_get_navigation_box (header_bar->action_bar_start);
+
+    g_settings_bind (EPHY_SETTINGS_WEB_APP, EPHY_PREFS_WEB_APP_MOBILE_CAPABLE, navigation_box, "visible", G_SETTINGS_BIND_DEFAULT | G_SETTINGS_BIND_INVERT_BOOLEAN);
+  }
 }
 
 static void
@@ -445,12 +411,37 @@ ephy_header_bar_set_adaptive_mode (EphyHeaderBar    *header_bar,
     case EPHY_ADAPTIVE_MODE_NORMAL:
       gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->start_revealer), TRUE);
       gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->end_revealer), TRUE);
+      gtk_widget_set_visible (header_bar->combined_stop_reload_button, FALSE);
 
       break;
     case EPHY_ADAPTIVE_MODE_NARROW:
       gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->start_revealer), FALSE);
       gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->end_revealer), FALSE);
+      gtk_widget_set_visible (header_bar->combined_stop_reload_button, TRUE);
 
       break;
   }
+
+  if (ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) != EPHY_EMBED_SHELL_MODE_APPLICATION)
+    ephy_location_entry_set_mobile_popdown (EPHY_LOCATION_ENTRY (header_bar->title_widget), adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW);
+}
+
+void
+ephy_header_bar_start_change_combined_stop_reload_state (EphyHeaderBar *header_bar,
+                                                         gboolean       loading)
+{
+  if (loading) {
+    gtk_image_set_from_icon_name (GTK_IMAGE (header_bar->combined_stop_reload_image),
+                                  "process-stop-symbolic",
+                                  get_icon_size ());
+    /* Translators: tooltip for the stop button */
+    gtk_widget_set_tooltip_text (header_bar->combined_stop_reload_button,
+                                 _("Stop loading the current page"));
+  } else {
+    gtk_image_set_from_icon_name (GTK_IMAGE (header_bar->combined_stop_reload_image),
+                                  "view-refresh-symbolic",
+                                  get_icon_size ());
+    gtk_widget_set_tooltip_text (header_bar->combined_stop_reload_button,
+                                 _(REFRESH_BUTTON_TOOLTIP));
+  }
 }
