@@ -37,7 +37,6 @@
 #include <gtk/gtk.h>
 #define HANDY_USE_UNSTABLE_API
 #include <handy.h>
-#include <libnotify/notify.h>
 #include <libxml/xmlreader.h>
 #include <libxml/xmlversion.h>
 #include <signal.h>
@@ -83,10 +82,10 @@ handle_shutdown_signal (gpointer user_data)
 }
 
 static gboolean
-application_mode_cb (const gchar *option_name,
-                     const gchar *value,
-                     gpointer     data,
-                     GError     **error)
+application_mode_cb (const gchar  *option_name,
+                     const gchar  *value,
+                     gpointer      data,
+                     GError      **error)
 {
   application_mode = TRUE;
   desktop_file_basename = g_strdup (value);
@@ -94,10 +93,10 @@ application_mode_cb (const gchar *option_name,
 }
 
 static gboolean
-option_version_cb (const gchar *option_name,
-                   const gchar *value,
-                   gpointer     data,
-                   GError     **error)
+option_version_cb (const gchar  *option_name,
+                   const gchar  *value,
+                   gpointer      data,
+                   GError      **error)
 {
   g_print ("%s %s\n", _("Web"), VERSION);
 
@@ -106,8 +105,7 @@ option_version_cb (const gchar *option_name,
 }
 
 /* If you're modifying this array then you need to update the manpage. */
-static const GOptionEntry option_entries[] =
-{
+static const GOptionEntry option_entries[] = {
   { "new-window", 0, 0, G_OPTION_ARG_NONE, &open_in_new_window,
     N_("Open a new browser window instead of a new tab"), NULL },
   { "load-session", 'l', 0, G_OPTION_ARG_FILENAME, &session_filename,
@@ -192,8 +190,6 @@ main (int   argc,
    */
   LIBXML_TEST_VERSION;
 
-  notify_init ("epiphany");
-
   /* If we're given -remote arguments, translate them */
   if (argc >= 2 && strcmp (argv[1], "-remote") == 0) {
     const char *opening, *closing;
@@ -317,8 +313,8 @@ main (int   argc,
   if (incognito_mode && profile_directory == NULL)
     profile_directory = ephy_default_profile_dir ();
 
-  /* Start our services */
-  flags = !application_mode ? EPHY_FILE_HELPERS_ENSURE_EXISTS : 0;
+  /* Required for bubblewrap sandbox. */
+  flags = EPHY_FILE_HELPERS_ENSURE_EXISTS;
 
   if (incognito_mode || private_instance || application_mode || automation_mode)
     flags |= EPHY_FILE_HELPERS_PRIVATE_PROFILE;
@@ -332,12 +328,12 @@ main (int   argc,
   }
 
   if (g_settings_get_boolean (EPHY_SETTINGS_MAIN, EPHY_PREFS_START_IN_INCOGNITO_MODE)) {
-     incognito_mode = TRUE;
+    incognito_mode = TRUE;
   }
 
   /* Run the migration in all cases, except when running a private
-     instance without a given profile directory or running in
-     incognito or automation mode. */
+   *  instance without a given profile directory or running in
+   *  incognito or automation mode. */
   if (!(private_instance && profile_directory == NULL) && !incognito_mode && !automation_mode) {
     /* If the migration fails we don't really want to continue. */
     if (!ephy_profile_utils_do_migration ((const char *)profile_directory, -1, FALSE)) {
@@ -433,9 +429,6 @@ main (int   argc,
   g_free (desktop_file_basename);
   g_free (profile_directory);
 
-  if (notify_is_initted ())
-    notify_uninit ();
-
   ephy_settings_shutdown ();
   ephy_file_helpers_shutdown ();
   xmlCleanupParser ();
