@@ -40,6 +40,7 @@
 #include "ephy-title-box.h"
 #include "ephy-title-widget.h"
 #include "ephy-type-builtins.h"
+#include "ephy-user-agent.h"
 #include "ephy-web-view.h"
 #include "ephy-window.h"
 #include "prefs-dialog.h"
@@ -124,8 +125,9 @@ ephy_shell_startup_continue (EphyShell               *shell,
     g_assert (session != NULL);
     ephy_session_load (session, (const char *)ctx->session_filename,
                        ctx->user_time, NULL, NULL, NULL);
-  } else if (new_window_option) {
-    static const char *default_uris[] = { "", NULL };
+  } else if (new_window_option && shell->remote_startup_context) {
+    char *homepage_url = g_settings_get_string (EPHY_SETTINGS_MAIN, EPHY_PREFS_HOMEPAGE_URL);
+    const char *default_uris[] = { homepage_url, NULL };
     const char **uris = NULL;
 
     if (ctx->arguments)
@@ -136,6 +138,7 @@ ephy_shell_startup_continue (EphyShell               *shell,
       uris = default_uris;
 
     ephy_shell_open_uris (shell, uris, ctx->startup_mode, ctx->user_time);
+    g_free (homepage_url);
   } else if (active_window && !ctx->arguments) {
     /* If the application already has an active window and the --new-window */
     /* option was not passed, then we should just present it */
@@ -287,6 +290,21 @@ show_downloads (GSimpleAction *action,
   g_signal_emit_by_name (manager, "show-downloads", NULL);
 }
 
+static void
+launch_app (GSimpleAction *action,
+            GVariant      *parameter,
+            gpointer       user_data)
+{
+  const gchar *desktop_file = g_variant_get_string (parameter, NULL);
+
+  /* We can't get here under flatpak because all web app functionality
+   * is disabled when running under flatpak.
+   */
+  ephy_file_launch_desktop_file (desktop_file,
+                                 gtk_get_current_event_time (),
+                                 EPHY_FILE_HELPERS_I_UNDERSTAND_I_MUST_NOT_USE_THIS_FUNCTION_UNDER_FLATPAK);
+}
+
 static GActionEntry app_entries[] = {
   { "new-window", new_window, NULL, NULL, NULL },
   { "new-incognito", new_incognito_window, NULL, NULL, NULL },
@@ -299,6 +317,7 @@ static GActionEntry app_entries[] = {
   { "about", show_about, NULL, NULL, NULL },
   { "quit", quit_application, NULL, NULL, NULL },
   { "show-downloads", show_downloads, NULL, NULL, NULL },
+  { "launch-app", launch_app, "s", NULL, NULL },
 };
 
 static GActionEntry non_incognito_extra_app_entries[] = {
@@ -398,6 +417,7 @@ ephy_shell_startup (GApplication *application)
   G_APPLICATION_CLASS (ephy_shell_parent_class)->startup (application);
 
   /* We're not remoting; start our services */
+  ephy_user_agent_init_sync ();
 
   mode = ephy_embed_shell_get_mode (embed_shell);
   if (mode != EPHY_EMBED_SHELL_MODE_APPLICATION) {
