@@ -56,6 +56,7 @@ void              ephy_window_set_zoom            (EphyWindow *window,
                                                    double zoom);
 
 void              ephy_window_activate_location   (EphyWindow *window);
+void              ephy_window_location_search     (EphyWindow *window);
 const char       *ephy_window_get_location        (EphyWindow *window);
 
 GtkWidget        *ephy_window_get_header_bar      (EphyWindow *window);
@@ -87,4 +88,11 @@ int               ephy_window_get_position_for_new_embed (EphyWindow *window,
 void              ephy_window_update_entry_focus         (EphyWindow  *window,
                                                           EphyWebView *view);
 
+gboolean          ephy_window_is_maximized               (EphyWindow *window);
+
+gboolean          ephy_window_is_fullscreen              (EphyWindow *window);
+
+void              ephy_window_get_geometry               (EphyWindow   *window,
+                                                          GdkRectangle *rectangle);
+
 G_END_DECLS
