@@ -60,7 +60,6 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libnotify/notify.h>
 #include <libsoup/soup.h>
 #include <string.h>
 #include <webkit2/webkit2.h>
@@ -93,9 +92,28 @@ window_cmd_new_incognito_window (GSimpleAction *action,
   ephy_open_incognito_window (NULL);
 }
 
-const gchar *import_option_names[2] = {
-  N_("GVDB File"),
-  N_("Firefox")
+typedef enum {
+  IMPORT_TYPE_CHOOSE,
+  IMPORT_TYPE_IMPORT
+} ImportTypes;
+
+
+struct import_option {
+  const char *name;
+  ImportTypes type;
+  gboolean (*exists)(void);
+};
+
+static gboolean firefox_exists (void);
+static gboolean chrome_exists (void);
+static gboolean chromium_exists (void);
+
+static struct import_option import_options[] = {
+  { N_("GVDB File"), IMPORT_TYPE_CHOOSE, NULL },
+  { N_("HTML File"), IMPORT_TYPE_CHOOSE, NULL },
+  { N_("Firefox"), IMPORT_TYPE_IMPORT, firefox_exists },
+  { N_("Chrome"), IMPORT_TYPE_IMPORT, chrome_exists },
+  { N_("Chromium"), IMPORT_TYPE_IMPORT, chromium_exists }
 };
 
 static void
@@ -108,9 +126,9 @@ combo_box_changed_cb (GtkComboBox *combo_box,
   g_assert (GTK_IS_BUTTON (button));
 
   active = gtk_combo_box_get_active (combo_box);
-  if (active == 0)
-    gtk_button_set_label (button, _("Ch_oose File…"));
-  else if (active == 1)
+  if (import_options[active].type == IMPORT_TYPE_CHOOSE)
+    gtk_button_set_label (button, _("Ch_oose File"));
+  else if (import_options[active].type == IMPORT_TYPE_IMPORT)
     gtk_button_set_label (button, _("I_mport"));
 }
 
@@ -181,6 +199,39 @@ get_firefox_profiles (void)
   return profiles;
 }
 
+static gboolean
+firefox_exists (void)
+{
+  GSList *firefox_profiles;
+  gboolean has_firefox_profile;
+
+  firefox_profiles = get_firefox_profiles ();
+  has_firefox_profile = g_slist_length (firefox_profiles) > 0;
+  g_slist_free (firefox_profiles);
+
+  return has_firefox_profile;
+}
+
+static gboolean
+chrome_exists (void)
+{
+  g_autofree char *filename = NULL;
+
+  filename = g_build_filename (g_get_user_config_dir (), "google-chrome", "Default", "Bookmarks", NULL);
+
+  return g_file_test (filename, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR);
+}
+
+static gboolean
+chromium_exists (void)
+{
+  g_autofree char *filename = NULL;
+
+  filename = g_build_filename (g_get_user_config_dir (), "chromium", "Default", "Bookmarks", NULL);
+
+  return g_file_test (filename, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR);
+}
+
 static GtkTreeModel *
 create_tree_model (void)
 {
@@ -189,27 +240,16 @@ create_tree_model (void)
   };
   GtkListStore *list_store;
   GtkTreeIter iter;
-  GSList *firefox_profiles;
-  gboolean has_firefox_profile;
   int i;
 
-
-  /* Check if user has a firefox profile*/
-  firefox_profiles = get_firefox_profiles ();
-  has_firefox_profile = g_slist_length (firefox_profiles) > 0;
-  g_slist_free (firefox_profiles);
-
   list_store = gtk_list_store_new (1, G_TYPE_STRING);
-  for (i = G_N_ELEMENTS (import_option_names) - 1; i >= 0; i--) {
-    /* Skip Firefox option if user doesn't have a Firefox profile */
-    if (g_strcmp0 (import_option_names[i], _("Firefox")) == 0) {
-      if (!has_firefox_profile)
-        continue;
-    }
+  for (i = G_N_ELEMENTS (import_options) - 1; i >= 0; i--) {
+    if (import_options[i].exists && !import_options[i].exists ())
+      continue;
 
     gtk_list_store_prepend (list_store, &iter);
     gtk_list_store_set (list_store, &iter,
-                        TEXT_COL, _(import_option_names[i]),
+                        TEXT_COL, _(import_options[i].name),
                         -1);
   }
 
@@ -275,98 +315,218 @@ show_profile_selector (GtkWidget *parent,
   return selected_profile;
 }
 
+gboolean
+dialog_bookmarks_import (GtkDialog *dialog)
+{
+  EphyBookmarksManager *manager = ephy_shell_get_bookmarks_manager (ephy_shell_get_default ());
+  g_autoptr (GtkFileChooserNative) file_chooser_dialog = NULL;
+  GtkWidget *import_info_dialog;
+  GtkFileFilter *filter;
+  int chooser_response;
+  gboolean imported = FALSE;
+
+  file_chooser_dialog = gtk_file_chooser_native_new (_("Choose File"),
+                                                     GTK_WINDOW (dialog),
+                                                     GTK_FILE_CHOOSER_ACTION_OPEN,
+                                                     _("I_mport"),
+                                                     _("_Cancel"));
+
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_add_pattern (filter, "*.gvdb");
+  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (file_chooser_dialog), filter);
+
+  chooser_response = gtk_native_dialog_run (GTK_NATIVE_DIALOG (file_chooser_dialog));
+  if (chooser_response == GTK_RESPONSE_ACCEPT) {
+    g_autoptr (GError) error = NULL;
+    g_autofree char *filename = NULL;
+
+    gtk_native_dialog_hide (GTK_NATIVE_DIALOG (file_chooser_dialog));
+
+    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser_dialog));
+    imported = ephy_bookmarks_import (manager, filename, &error);
+
+    import_info_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
+                                                 GTK_DIALOG_MODAL,
+                                                 imported ? GTK_MESSAGE_INFO : GTK_MESSAGE_WARNING,
+                                                 GTK_BUTTONS_OK,
+                                                 "%s",
+                                                 imported ? _("Bookmarks successfully imported!")
+                                                          : error->message);
+    gtk_dialog_run (GTK_DIALOG (import_info_dialog));
+
+    gtk_widget_destroy (import_info_dialog);
+  }
+  return imported;
+}
+
+gboolean
+dialog_bookmarks_import_from_html (GtkDialog *dialog)
+{
+  EphyBookmarksManager *manager = ephy_shell_get_bookmarks_manager (ephy_shell_get_default ());
+  g_autoptr (GtkFileChooserNative) file_chooser_dialog = NULL;
+  GtkWidget *import_info_dialog;
+  GtkFileFilter *filter;
+  int chooser_response;
+  gboolean imported = FALSE;
+
+  file_chooser_dialog = gtk_file_chooser_native_new (_("Choose File"),
+                                                     GTK_WINDOW (dialog),
+                                                     GTK_FILE_CHOOSER_ACTION_OPEN,
+                                                     _("I_mport"),
+                                                     _("_Cancel"));
+
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_add_pattern (filter, "*.html");
+  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (file_chooser_dialog), filter);
+
+  chooser_response = gtk_native_dialog_run (GTK_NATIVE_DIALOG (file_chooser_dialog));
+  if (chooser_response == GTK_RESPONSE_ACCEPT) {
+    g_autoptr (GError) error = NULL;
+    g_autofree char *filename = NULL;
+
+    gtk_native_dialog_hide (GTK_NATIVE_DIALOG (file_chooser_dialog));
+
+    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser_dialog));
+    imported = ephy_bookmarks_import_from_html (manager, filename, &error);
+
+    import_info_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
+                                                 GTK_DIALOG_MODAL,
+                                                 imported ? GTK_MESSAGE_INFO : GTK_MESSAGE_WARNING,
+                                                 GTK_BUTTONS_OK,
+                                                 "%s",
+                                                 imported ? _("Bookmarks successfully imported!")
+                                                          : error->message);
+    gtk_dialog_run (GTK_DIALOG (import_info_dialog));
+
+    gtk_widget_destroy (import_info_dialog);
+  }
+  return imported;
+}
+
+gboolean
+dialog_bookmarks_import_from_firefox (GtkDialog *dialog)
+{
+  EphyBookmarksManager *manager = ephy_shell_get_bookmarks_manager (ephy_shell_get_default ());
+  GtkWidget *import_info_dialog;
+  g_autoptr (GError) error = NULL;
+  GSList *profiles;
+  g_autofree gchar *profile = NULL;
+  int num_profiles;
+  gboolean imported = FALSE;
+
+  profiles = get_firefox_profiles ();
+
+  /* Import default profile */
+  num_profiles = g_slist_length (profiles);
+  if (num_profiles == 1) {
+    imported = ephy_bookmarks_import_from_firefox (manager, profiles->data, &error);
+  } else if (num_profiles > 1) {
+    profile = show_profile_selector (GTK_WIDGET (dialog), profiles);
+    if (profile) {
+      imported = ephy_bookmarks_import_from_firefox (manager, profile, &error);
+    }
+  } else {
+    g_assert_not_reached ();
+  }
+
+  g_slist_free_full (profiles, g_free);
+
+  /* If there are multiple profiles, but the user didn't select one in
+   * the profile (he pressed Cancel), don't display the import info dialog
+   * as no import took place
+   */
+  if (num_profiles == 1 || profile) {
+    import_info_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
+                                                 GTK_DIALOG_MODAL,
+                                                 imported ? GTK_MESSAGE_INFO : GTK_MESSAGE_WARNING,
+                                                 GTK_BUTTONS_OK,
+                                                 "%s",
+                                                 imported ? _("Bookmarks successfully imported!")
+                                                          : error->message);
+    gtk_dialog_run (GTK_DIALOG (import_info_dialog));
+    gtk_widget_destroy (import_info_dialog);
+  }
+  return imported;
+}
+
+static gboolean
+dialog_bookmarks_import_from_chrome (GtkDialog *dialog)
+{
+  EphyBookmarksManager *manager = ephy_shell_get_bookmarks_manager (ephy_shell_get_default ());
+  GtkWidget *import_info_dialog;
+  g_autoptr (GError) error = NULL;
+  g_autofree gchar *filename;
+  gboolean imported;
+
+  filename = g_build_filename (g_get_user_config_dir (), "google-chrome", "Default", "Bookmarks", NULL);
+
+  imported = ephy_bookmarks_import_from_chrome (manager, filename, &error);
+  import_info_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
+                                               GTK_DIALOG_MODAL,
+                                               imported ? GTK_MESSAGE_INFO : GTK_MESSAGE_WARNING,
+                                               GTK_BUTTONS_OK,
+                                               "%s",
+                                               imported ? _("Bookmarks successfully imported!")
+                                                        : error->message);
+  gtk_dialog_run (GTK_DIALOG (import_info_dialog));
+  gtk_widget_destroy (import_info_dialog);
+
+  return imported;
+}
+
+static gboolean
+dialog_bookmarks_import_from_chromium (GtkDialog *dialog)
+{
+  EphyBookmarksManager *manager = ephy_shell_get_bookmarks_manager (ephy_shell_get_default ());
+  GtkWidget *import_info_dialog;
+  g_autoptr (GError) error = NULL;
+  g_autofree gchar *filename;
+  gboolean imported;
+
+  filename = g_build_filename (g_get_user_config_dir (), "chromium", "Default", "Bookmarks", NULL);
+
+  imported = ephy_bookmarks_import_from_chrome (manager, filename, &error);
+  import_info_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
+                                               GTK_DIALOG_MODAL,
+                                               imported ? GTK_MESSAGE_INFO : GTK_MESSAGE_WARNING,
+                                               GTK_BUTTONS_OK,
+                                               "%s",
+                                               imported ? _("Bookmarks successfully imported!")
+                                                        : error->message);
+  gtk_dialog_run (GTK_DIALOG (import_info_dialog));
+  gtk_widget_destroy (import_info_dialog);
+
+  return imported;
+}
+
 static void
 dialog_bookmarks_import_cb (GtkDialog   *dialog,
                             int          response,
                             GtkComboBox *combo_box)
 {
-  EphyBookmarksManager *manager = ephy_shell_get_bookmarks_manager (ephy_shell_get_default ());
-  GtkFileChooser *file_chooser_dialog;
-  GtkWidget *import_info_dialog;
   int active;
-  int chooser_response;
   gboolean imported = FALSE;
 
   if (response == GTK_RESPONSE_OK) {
     active = gtk_combo_box_get_active (combo_box);
-    if (active == 0) {
-      GtkFileFilter *filter;
-
-      file_chooser_dialog = GTK_FILE_CHOOSER (gtk_file_chooser_native_new (_("Choose File"),
-                                                                           GTK_WINDOW (dialog),
-                                                                           GTK_FILE_CHOOSER_ACTION_OPEN,
-                                                                           _("I_mport"),
-                                                                           _("_Cancel")));
-      gtk_file_chooser_set_show_hidden (file_chooser_dialog, TRUE);
-
-      filter = gtk_file_filter_new ();
-      gtk_file_filter_add_pattern (filter, "*.gvdb");
-      gtk_file_chooser_set_filter (file_chooser_dialog, filter);
-
-      chooser_response = gtk_native_dialog_run (GTK_NATIVE_DIALOG (file_chooser_dialog));
-      if (chooser_response == GTK_RESPONSE_ACCEPT) {
-        GError *error = NULL;
-        char *filename;
-
-        gtk_native_dialog_hide (GTK_NATIVE_DIALOG (file_chooser_dialog));
-
-        filename = gtk_file_chooser_get_filename (file_chooser_dialog);
-        imported = ephy_bookmarks_import (manager, filename, &error);
-        g_free (filename);
-
-        import_info_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
-                                                     GTK_DIALOG_MODAL,
-                                                     imported ? GTK_MESSAGE_INFO : GTK_MESSAGE_WARNING,
-                                                     GTK_BUTTONS_OK,
-                                                     "%s",
-                                                     imported ? _("Bookmarks successfully imported!")
-                                                              : error->message);
-        gtk_dialog_run (GTK_DIALOG (import_info_dialog));
-
-        gtk_widget_destroy (import_info_dialog);
-      }
-      g_object_unref (file_chooser_dialog);
-    } else if (active == 1) {
-      GError *error = NULL;
-      GSList *profiles;
-      gchar *profile = NULL;
-      int num_profiles;
-
-      profiles = get_firefox_profiles ();
-
-      /* Import default profile */
-      num_profiles = g_slist_length (profiles);
-      if (num_profiles == 1) {
-        imported = ephy_bookmarks_import_from_firefox (manager, profiles->data, &error);
-      } else if (num_profiles > 1) {
-        profile = show_profile_selector (GTK_WIDGET (dialog), profiles);
-        if (profile) {
-          imported = ephy_bookmarks_import_from_firefox (manager, profile, &error);
-          g_free (profile);
-        }
-      } else {
+    switch (active) {
+      case 0:
+        imported = dialog_bookmarks_import (dialog);
+        break;
+      case 1:
+        imported = dialog_bookmarks_import_from_html (dialog);
+        break;
+      case 2:
+        imported = dialog_bookmarks_import_from_firefox (dialog);
+        break;
+      case 3:
+        imported = dialog_bookmarks_import_from_chrome (dialog);
+        break;
+      case 4:
+        imported = dialog_bookmarks_import_from_chromium (dialog);
+        break;
+      default:
         g_assert_not_reached ();
-      }
-
-      g_slist_free (profiles);
-
-      /* If there are multiple profiles, but the user didn't select one in
-       * the profile (he pressed Cancel), don't display the import info dialog
-       * as no import took place
-       */
-      if (num_profiles == 1 || profile) {
-        import_info_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
-                                                     GTK_DIALOG_MODAL,
-                                                     imported ? GTK_MESSAGE_INFO : GTK_MESSAGE_WARNING,
-                                                     GTK_BUTTONS_OK,
-                                                     "%s",
-                                                     imported ? _("Bookmarks successfully imported!")
-                                                              : error->message);
-        gtk_dialog_run (GTK_DIALOG (import_info_dialog));
-        gtk_widget_destroy (import_info_dialog);
-      }
-      if (error)
-        g_error_free (error);
     }
 
     if (imported)
@@ -550,6 +710,9 @@ window_cmd_show_shortcuts (GSimpleAction *action,
     builder = gtk_builder_new_from_resource ("/org/gnome/epiphany/gtk/shortcuts-dialog.ui");
     shortcuts_window = GTK_WIDGET (gtk_builder_get_object (builder, "shortcuts-dialog"));
 
+    if (ephy_is_running_inside_flatpak ())
+      gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "shortcuts-web-apps-group")));
+
     if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL) {
       GtkShortcutsShortcut *shortcut;
 
@@ -964,6 +1127,7 @@ typedef struct {
   char *icon_href;
   GdkRGBA icon_rgba;
   GCancellable *cancellable;
+  gboolean mobile_capable;
   WebKitDownload *download;
 } EphyApplicationDialogData;
 
@@ -1288,18 +1452,23 @@ fill_default_application_title (EphyApplicationDialogData *data)
 }
 
 static void
-notify_launch_cb (NotifyNotification *notification,
-                  char               *action,
-                  gpointer            user_data)
+fill_mobile_capable_cb (GObject      *source,
+                        GAsyncResult *async_result,
+                        gpointer      user_data)
 {
-  g_autofree char *desktop_file = user_data;
+  g_autoptr (GError) error = NULL;
+  EphyApplicationDialogData *data = user_data;
+  gboolean capable;
 
-  /* We can't get here under flatpak because all web app functionality
-   * is disabled when running under flatpak.
-   */
-  ephy_file_launch_desktop_file (desktop_file,
-                                 gtk_get_current_event_time (),
-                                 EPHY_FILE_HELPERS_I_UNDERSTAND_I_MUST_NOT_USE_THIS_FUNCTION_UNDER_FLATPAK);
+  capable = ephy_web_view_get_web_app_mobile_capable_finish (EPHY_WEB_VIEW (source), async_result, &error);
+  if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+    data->mobile_capable = capable;
+}
+
+static void
+fill_mobile_capable (EphyApplicationDialogData *data)
+{
+  ephy_web_view_get_web_app_mobile_capable (data->view, data->cancellable, fill_mobile_capable_cb, data);
 }
 
 static gboolean
@@ -1393,8 +1562,8 @@ dialog_save_as_application_response_cb (GtkDialog                 *dialog,
     const char *app_name;
     g_autofree gchar *app_id = NULL;
     g_autofree gchar *desktop_file = NULL;
-    char *message;
-    NotifyNotification *notification;
+    g_autofree char *message = NULL;
+    GNotification *notification;
 
     app_name = gtk_entry_get_text (GTK_ENTRY (data->entry));
     app_id = ephy_web_application_get_app_id_from_name (app_name);
@@ -1410,7 +1579,8 @@ dialog_save_as_application_response_cb (GtkDialog                 *dialog,
     desktop_file = ephy_web_application_create (app_id,
                                                 webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view)),
                                                 app_name,
-                                                gtk_image_get_pixbuf (GTK_IMAGE (data->image)));
+                                                gtk_image_get_pixbuf (GTK_IMAGE (data->image)),
+                                                data->mobile_capable);
 
     if (desktop_file)
       message = g_strdup_printf (_("The application “%s” is ready to be used"),
@@ -1419,25 +1589,27 @@ dialog_save_as_application_response_cb (GtkDialog                 *dialog,
       message = g_strdup_printf (_("The application “%s” could not be created"),
                                  app_name);
 
-    notification = notify_notification_new (message,
-                                            NULL, NULL);
-    g_free (message);
+    notification = g_notification_new (message);
+
+    if (data->image) {
+      GdkPixbuf *pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (data->image));
+
+      g_notification_set_icon (notification, G_ICON (pixbuf));
+    }
 
     if (desktop_file) {
-      notify_notification_add_action (notification, "launch", _("Launch"),
-                                      (NotifyActionCallback)notify_launch_cb,
-                                      g_path_get_basename (desktop_file),
-                                      NULL);
-      notify_notification_set_icon_from_pixbuf (notification, gtk_image_get_pixbuf (GTK_IMAGE (data->image)));
+      g_autofree char *basename = g_path_get_basename (desktop_file);
+
+      /* Translators: Desktop notification when a new web app is created. */
+      g_notification_add_button_with_target (notification, _("Launch"), "app.launch-app", "s", basename);
+      g_notification_set_default_action_and_target (notification, "app.launch-app", "s", basename);
 
       ephy_focus_desktop_app (desktop_file);
     }
 
-    notify_notification_set_timeout (notification, NOTIFY_EXPIRES_DEFAULT);
-    notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
-    notify_notification_set_hint (notification, "desktop-entry", g_variant_new_string ("epiphany"));
-    notify_notification_set_hint (notification, "transient", g_variant_new_boolean (TRUE));
-    notify_notification_show (notification, NULL);
+    g_notification_set_priority (notification, G_NOTIFICATION_PRIORITY_LOW);
+
+    g_application_send_notification (G_APPLICATION (g_application_get_default ()), app_name, notification);
   }
 
   ephy_application_dialog_data_free (data);
@@ -1529,6 +1701,7 @@ window_cmd_save_as_application (GSimpleAction *action,
 
   fill_default_application_image (data);
   fill_default_application_title (data);
+  fill_mobile_capable (data);
 
   gtk_widget_show_all (dialog);
 
@@ -2122,6 +2295,14 @@ window_cmd_go_location (GSimpleAction *action,
   ephy_window_activate_location (user_data);
 }
 
+void
+window_cmd_location_search (GSimpleAction *action,
+                            GVariant      *parameter,
+                            gpointer       user_data)
+{
+  ephy_window_location_search (user_data);
+}
+
 void
 window_cmd_go_home (GSimpleAction *action,
                     GVariant      *parameter,
